#!/bin/bash

## Declarar variables
CONF_FILE=/etc/linkat/linkat-client/linkat-client.conf
PLANTILLES=/usr/share/linkat/linkat-client/plantilles
FILES_LINKAT=/usr/share/linkat/linkat-client/configurador/files
ANSIBLEPLAY=/usr/share/linkat/linkat-client/configurador
LOGS=/var/log/linkat-client.log
DATE=`date '+%Y-%m-%d_%H:%M:%S'`

if [ -f "$CONF_FILE" ]; then
  . "$CONF_FILE"
else
  NEW_DOMAIN="intracentre"
  NEW_SERVIDOR_IP="192.168.0.240"
#  NEW_PASSLNADMIN1=""
#  NEW_PASSLNADMIN2=""
fi

## Revisar valor de xarxa
check_ip()
{
        echo "$2" | grep -E '^(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])$' > /dev/null 2>&1
        if [ "$?" -gt 0 ]; then
		yad --title="Error" --text="\nEl valor $1: $2 no és vàlid." --image="dialog-error" --button="D'acord"
		ERROR="1"
	fi
}

## Revisar connexió
check_connexio()
{
	sudo hping3 -S "$NEW_SERVIDOR_IP" -p 389 -c 4
	if [ $? -eq 0 ]; then
		CONNEXIO="1"
	else
		echo -en "Error de connexió: No s'ha trobat el servidor LDAP"
		exit 12
	fi
}

## Formulari de dades de configuracions del client de centre

formulari()
{
res=$(yad --width=400 --title="Linkat Client de centre" --text="\nIntroduïu els valors per configurar el client de centre.\nCal emplenar tots els camps.\n\nConfiguracions del client:\n" \
--image="/usr/share/linkat/linkat-client/linkat-client-banner.png" \
--form --item-separator=" " \
--field="Nom del domini" \
--field="IP del servidor" \
--button="D'acord" --button="Cancel·la":11 \
"$NEW_DOMAIN" "$NEW_SERVIDOR_IP")

res1="$?"

if [ "$res1" -gt 1 ]; then
	exit 1
fi

NEW_DOMAIN=$(echo "$res" | awk -F"|" '{print $1}')
NEW_SERVIDOR_IP=$(echo "$res" | awk -F"|" '{print $2}')
}

check_errors()
{
if [ ! "$?" -eq 0 ]; then
	echo -en "Error: $1"
	yad --title="Error" --text="\nS'ha produit un error durant la instal·lacio de: $1.\nEl programa es tancara." --image="dialog-error" --button="D'acord"
	exit 22
fi
}

move_homes()
{
if [ ! -f /etc/modalitat_linkat ] ; then
	if [ ! -d /home-local ] ; then
                mkdir /home-local/
        fi
        for users in $(ls /home)
        do
                mv /home/"$users" /home-local/
                sed -i "s/\/home\/$users/\/home-local\/$users/g" /etc/passwd
	done
	sed -i 's/\/home/\/home-local/g' /etc/default/useradd		
	echo client > /etc/modalitat_linkat
fi
}

ERROR="1"

while [ "$ERROR" -eq "1" ]
do
	ERROR="0"
	formulari
	check_ip IP "$NEW_SERVIDOR_IP"
done

## Backup del fitxer de configuració linkat-client.conf
if [ -f "$CONF_FILE" ]; then
	cp -av "$CONF_FILE" "$CONF_FILE"."$DATE"
fi

## Genera nou fitxer de configuració linkat-client.conf
echo "$DATE" > $CONF_FILE
echo "NEW_DOMAIN=$NEW_DOMAIN" >> $CONF_FILE
echo "NEW_SERVIDOR_IP=$NEW_SERVIDOR_IP" >> $CONF_FILE

## Copia plantilles per modificar
rm -rf "$FILES_LINKAT"/*
cp -av "$PLANTILLES"/* "$FILES_LINKAT"/

cd "$FILES_LINKAT"/

## Genera unitats pam_mount
RECURSOS="$(smbclient -L $NEW_SERVIDOR_IP -g -N 2>/dev/null|grep -i ^disk|awk -F '|' '{print $2}')"

if [ -f volume-final ] ; then
        rm volume-final
fi
for i in $RECURSOS
        do cat volume | sed s/__RECURS__/$i/g >> volume-final
done

sed -i '/<pam_mount>/r volume-final' pam_mount.conf.xml

## Modifica variables

sed -i s/__DOMAIN__/"$NEW_DOMAIN"/g *
sed -i s/__IP__/"$NEW_SERVIDOR_IP"/g *

echo "$DATE" >> "$LOGS"

## Aplica configuracions
echo -en "Aplicant configuracions...\n\n"

killall update-manager update-notifier 2>&1
dpkg -s auth-client-config > /dev/null 2>&1
res="$?"
if [ "$res" -eq 0 ]; then
        sudo apt purge libnss-ldap auth-client-config libpam-ldapd libnss-ldapd nss-updatedb libnss-db nscd nslcd ldap-utils  -y
fi

sudo debconf-set-selections < "$FILES_LINKAT"/debconf-ldap-auth-config
sudo debconf-set-selections < "$FILES_LINKAT"/debconf-nslcd
sudo apt install nslcd nscd ldap-utils libpam-ldapd nss-updatedb libnss-ldap auth-client-config -y

## Aplica ANSIBLE
ansible-playbook "$ANSIBLEPLAY"/client.yml

#echo "session required pam_mkhomedir.so skel=/etc/skel umask=0002" >> /etc/pam.d/login
#echo "session required pam_mkhomedir.so skel=/etc/skel umask=0002" >> /etc/pam.d/lightdm

sudo auth-client-config -t nss -p lac_ldap
sudo pam-auth-update

## Revisa connexió
#check_connexio

yad --width=300 --title="Linkat Client de centre" --text="\nLa configuració ha estat aplicada.\n\nEl client de centre s'ha de reiniciar per aplicar els canvis.\n\nVoleu reiniciar-lo ara?" \
--image="/usr/share/linkat/linkat-client/linkat-client-banner.png" \
--button="D'acord" --button="Cancel·la":11

if [ $? -eq 0 ]; then
	move_homes
	sudo shutdown -r now
else
	move_homes
fi
